package ece476controlmapper;

import java.awt.AWTException;
import java.awt.Robot;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Hashtable;
import java.util.List;
import java.util.Map;

import javax.swing.Timer;

public class ControlMapper implements SerialInputLineHandler {

	private Hashtable<String, GestureControl> gestures = new Hashtable<String, GestureControl>();
	ControlMapperGUI gui;
	Robot robot;

	public ControlMapper(ControlMapperGUI gui) {
		this.gui = gui;
		try {
			robot = new Robot();
		} catch (AWTException e) {
			e.printStackTrace();
		}
	}

	public void register(GestureControl g) {
		gestures.put(g.getId(), g);
	}

	public GestureControl getGesture(String id) {
		return gestures.get(id);
	}

	public List<GestureControl> getGestureList() {
		ArrayList<GestureControl> s = new ArrayList<GestureControl>(gestures.size());
		for (Map.Entry<String, GestureControl> ge : gestures.entrySet()) {
			GestureControl g = ge.getValue();
			s.add(g);
		}
		Collections.sort(s);
		return s;
	}

	@Override
	public void handleInputLine(String line) {
		try {
			String gid = parse(line);
			GestureControl g = gestures.get(gid);
			if (g == null || !g.isEnabled()) {
				gui.println(line);
				return;
			} else {
				performAction(g);
			}
		} catch (Exception e) {
			gui.showErrorMessage(e.getMessage());
		} catch (Error e) {
			gui.showErrorMessage("Error! " + e.getMessage());
		}
	}

	private void performAction(GestureControl g) {
		if (robot == null)
			return;
		ArrayList<Integer> keycodes = g.getKey();
		switch (g.getActionType()) {
			case GestureControl.KEY_STROKE:
				for (Integer keycode : keycodes) {
				robot.keyPress(keycode);
				(new RobotTimer(robot, keycode, GestureControl.strokeTime))
						.start();
				gui.println("Key stroke: " + KeyEvent.getKeyText(keycode));
			}
			break;
			case GestureControl.KEY_PRESS:
				for (Integer keycode : keycodes) {
					robot.keyPress(keycode);
					gui.println("Key pressed: " + KeyEvent.getKeyText(keycode));
				}
				break;
			case GestureControl.KEY_RELEASE:
				for (Integer keycode : keycodes) {
					robot.keyRelease(keycode);
					gui.println("Key released: " + KeyEvent.getKeyText(keycode));
				}
				break;
			case GestureControl.KEY_LONG_STROKE:
				for (Integer keycode : keycodes) {
					robot.keyPress(keycode);
					(new RobotTimer(robot, keycode, GestureControl.longStrokeTime))
							.start();
					gui.println("Key stroke: " + KeyEvent.getKeyText(keycode));
				}
				break;
			default:
				break;
		}
	}

	public String parse(String line) {
		return line.trim();
	}

	public void load(File file) {
		gestures.clear();
		try {
			BufferedReader w = new BufferedReader(new FileReader(file));
			String line;
			while ((line = w.readLine()) != null) {
				line = line.replaceAll("\\[.*\\]", "").trim();
				if (line.length() == 0)
					continue;
				GestureControl g = GestureControl.fromString(line);
				if (g!=null) {
					register(g);
				}
				else {
					//gui.showErrorMessage("Error parsing mapping file " + file.getAbsolutePath());
				}
			}
			w.close();
		} catch (IOException e) {
			gui.showErrorMessage("Error reading mapping file: "
					+ file.getAbsolutePath());
		}
	}

	public void write(File file) {
		try {
			BufferedWriter w = new BufferedWriter(new FileWriter(file));
			w.write("");
			for (GestureControl g : getGestureList()) {
				w.append(GestureControl.toString(g) + "\n");
			}
			w.close();
		} catch (IOException e) {
			gui.showErrorMessage("Error writing mapping file: "
					+ file.getAbsolutePath());
		}
	}
	
	private static class RobotTimer extends Timer implements ActionListener {
		private static final long serialVersionUID = 1L;
		Robot robot;
		int keycode;
		public RobotTimer(Robot robot, int keycode, int delay) {
			super(delay, null);
			this.robot = robot;
			this.keycode = keycode;
			setRepeats(false);
			addActionListener(this);
		}
		@Override
		public void actionPerformed(ActionEvent e) {
			robot.keyRelease(keycode);
		}	
	}
}
